'
' Macros used in the Fixed Point FFT
'
' Note that none of the macros perform any RAM bank switching so ensure that all variables used are in Access RAM
'
' Bring some of the compiler's System Variables into the BASIC program for use with the Fixed Point pMultiplier
'
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim PP1 As Byte System
    Dim PP1H As Byte System
'--------------------------------------------------------------------------------------------------
' Signed Divide 16-bit Parameter By 2 (Var = pDividend / 2)
'
Div2 Macro pDividend
#if(Div2_Return == 0)                ; If no return variable given, do the following code
Asm-
    bcf STATUS,C
    rrcf pDividend + 1,f            ; Divide high byte
    rrcf pDividend,f                ; and low byte
    btfsc pDividend + 1,6           ; Test old sign bit
    bsf pDividend + 1,7             ; If set, alter the msb
Endasm-
#else
    #if(Return_Var == pDividend)    ; Is the return variable the same as the parameter ?
Asm-
    bcf STATUS,C
    rrcf pDividend + 1,f            ; Divide high byte
    rrcf pDividend,f                ; and low byte
    btfsc pDividend + 1,6           ; Test old sign bit
    bsf pDividend + 1,7             ; If set, alter the msb
EndAsm
    #else
Asm-
    movff pDividend,Return_Var
    movff pDividend + 1,Return_Var + 1
    bcf STATUS,C
    rrcf Return_Var + 1,f            ; Divide high byte
    rrcf Return_Var,f                ; and low byte
    btfsc Return_Var + 1,6           ; Test old sign bit
    bsf Return_Var + 1,7             ; If set, alter the msb
EndAsm
    #endif
#endif
    Endm
'--------------------------------------------------------------------------------------------------
' Signed 16-bit Multiply
' Notes     : Overflow not detected if Fixed Point result > 4.0
'
FixedMult Macro pMultiplier, pMultiplicand
#if(pMultiplier == pMultiplicand)                    ; pMultiplier and pMultiplicand are the same variable?
    #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32) || (Prm_1 == SNum8) || (Prm_1 == SNum16) || (Prm_1 == SNum32))
        Num_Word pMultiplier,PP0
        Num_Word pMultiplier,PP1
    #endif
    #if(Prm_1 == Byte)
Asm-
    movf pMultiplier,W
    movwf PP0
    movwf PP1
EndAsm
    #endif
    #if((Prm_1 == Word) || (Prm_1 == Dword))
Asm-
    movf pMultiplier,W
    movwf PP0
    movwf PP1
    movf pMultiplier + 1,W
    movwf PP0H
    movwf PP1H
EndAsm
    #endif
#else
    #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
        Num_Word pMultiplier,PP0
    #endif
    #if(Prm_1 == Byte)
        Byte_Word pMultiplier,PP0
    #endif
    #if((Prm_1 == Word) || (Prm_1 == Dword))
        Word_Word pMultiplier,PP0
    #endif
    #if((Prm_2 == Num8) || (Prm_2 == Num16) || (Prm_2 == Num32) || (Prm_2 == SNum8) || (Prm_2 == SNum16) || (Prm_2 == SNum32))
        Num_Word pMultiplicand,PP1
    #endif
    #if(Prm_2 == Byte)
        Byte_Word pMultiplicand,PP1
    #endif
    #if((Prm_2 == Word) || (Prm_2 == Dword))
        Word_Word pMultiplicand,PP1
    #endif
#endif
    GoSub FixedMultSub
Asm-
    movff MultTemp3, Return_Var + 1    ; Store the result..
    movff MultTemp2, Return_Var        ; in return variable
EndAsm
    Endm
'---------------------------------------------------------------------------
' Fixed Point Multiply Subroutine used by the FixedMult Macro
'
' Input     : PP0 = Multiplier
'             PP1 = Multiplicand
' Output    : MultTemp2 and MultTemp3 hold the 16-bit result
'
#ifdef FixedMult#req
FixedMultSub:
    movf PP0,w
    mulwf PP1
    movff PRODH,MultTemp1
    movf PP0H,w
    mulwf PP1H
    movff PRODL,MultTemp2
    movff PRODH,MultTemp3
    movf PP0,w
    mulwf PP1H
    movf PRODL,w
    addwf MultTemp1,f
    movf PRODH,w
    addwfc MultTemp2,f
    movlw 0
    addwfc MultTemp3,f
    movf PP0H,w
    mulwf PP1
    movf PRODL,w
    addwf MultTemp1,f
    movf PRODH,w
    addwfc MultTemp2,f
    movlw 0
    addwfc MultTemp3,f
    btfss PP1H,7      		; Is PP1 < 0?
    bra $ + 10
    movf PP0,w            	; Yes. So, adjust value
    subwf MultTemp2,f
    movf PP0H,w
    subwfb MultTemp3,f
    btfss PP0H,7            ; Is PP0 < 0?
    bra $ + 10
    movf PP1,w           	; Yes. So, adjust value
    subwf MultTemp2,f
    movf PP1H,w
    subwfb MultTemp3,f
    rlcf MultTemp1,f
    rlcf MultTemp2,f
    rlcf MultTemp3,f
    rlcf MultTemp1,f
    rlcf MultTemp2,f
    rlcf MultTemp3,f
    rlcf MultTemp1,f
    rlcf MultTemp2,f
    rlcf MultTemp3,f
    Return
#endif
'---------------------------------------------------------------------------
' Signed Fixed Point divide routine
'
' Input      : Arguments in AccumA and DivTemp
' Output     : Quotient AccumA
'
' Notes      : Rolled out for speed at the cost of code size
'            : Uses PRODL\H as temporary variables
'
Udiv1608  Macro
Asm-
    clrf PRODL
NOLIST
 Variable mDivEntities = 0
 While (mDivEntities < 8)
LIST
    rlcf AccumAH,w
    rlcf PRODL,f
    movf DivTemp,w
    subwf PRODL,f
    bc $ + 6
    addwf PRODL,f
    bcf STATUS,C
    rlcf AccumAH,f
NOLIST
 Variable mDivEntities = mDivEntities + 1
 endw
LIST
    clrf PRODH
NOLIST
 Variable mDivEntities = 8
 While mDivEntities < 16
LIST
    rlcf AccumA,w
    rlcf PRODL,f
    rlcf PRODH,f
    movf DivTemp,w
    subwf PRODL,f
    movlw 0
    subwfb PRODH,f
    bc $ + 12
    movf DivTemp,w
    addwf PRODL,f
    movlw 0
    addwfc PRODH,f
    bcf STATUS,C
    rlcf AccumA,f
NOLIST
 Variable mDivEntities = mDivEntities + 1
 endw
LIST
EndAsm
    Endm
'--------------------------------------------------------------------------------------------------

